/*
 * SonarQube
 * Copyright (C) 2009-2019 SonarSource SA
 * mailto:info AT sonarsource DOT com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package net.educoder.ecsonar.utils.html;


import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 给代码增加样式
 */
public final class HtmlSourceDecorator {

    private static volatile HtmlSourceDecorator htmlSourceDecorator = null;

    private HtmlSourceDecorator() {
    }

    public static HtmlSourceDecorator getInstance() {
        if (htmlSourceDecorator == null) {
            synchronized (HtmlSourceDecorator.class) {
                if (htmlSourceDecorator == null) {
                    htmlSourceDecorator = new HtmlSourceDecorator();
                }
            }
        }
        return htmlSourceDecorator;
    }

    public String getDecoratedSourceAsHtml(@NotNull String sourceLine, @NotNull String highlighting, @NotNull String symbols) {
        if (sourceLine == null) {
            return null;
        }
        DecorationDataHolder decorationDataHolder = new DecorationDataHolder();
        if (StringUtils.isNotBlank(highlighting)) {
            decorationDataHolder.loadSyntaxHighlightingData(highlighting);
        }
        if (StringUtils.isNotBlank(symbols)) {
            decorationDataHolder.loadLineSymbolReferences(symbols);
        }
        HtmlTextDecorator textDecorator = new HtmlTextDecorator();
        List<String> decoratedSource = textDecorator.decorateTextWithHtml(sourceLine, decorationDataHolder, 1, 1);
        if (decoratedSource == null) {
            return null;
        } else {
            if (decoratedSource.isEmpty()) {
                return "";
            } else {
                return decoratedSource.get(0);
            }
        }
    }


}
